import 'package:flutter/material.dart';
import 'package:flutterskel/provider/ProviderBuilder.dart' as arch;

import '../configuration/AppConfiguration.dart';
import '../configuration/ConfigurationRepository.dart';
import '../data/sources/DataSource.dart';
import '../resources/colors/AppColors.dart';
import '../resources/dimensions/AppDimensions.dart';
import '../resources/fonts/AppFonts.dart';
import '../resources/icons/AppIcons.dart';
import '../resources/images/AppImages.dart';
import '../resources/strings/AppStrings.dart';
import '../resources/strings/StringsRepository.dart';
import '../resources/themes/AppTheme.dart';
import '../resources/themes/ThemeRepository.dart';
import '../routing/page/PageRouter.dart';
import '../routing/url/UrlRouter.dart';
import '../userinformation/UserInformationRepository.dart';

class ProviderBuilder
    extends arch.ProviderBuilder<
        DataSource,
        // Routing
        UrlRouter,
        PageRouter,
        // Configuration
        ConfigurationRepository,
        UserInformationRepository,
        ThemeRepository,
        StringsRepository,
        AppImages,
        AppIcons> {

  @override
  ConfigurationRepository get configurationRepository {
    AppConfiguration configuration = AppConfiguration();
    return ConfigurationRepository(configuration: configuration);
  }

  @override
  DataSource get dataSource => DataSource();

  @override
  AppIcons get icons => AppIcons();

  @override
  AppImages get images => AppImages();

  @override
  PageRouter get pageRouter => PageRouter();

  @override
  StringsRepository get stringsRepository {
    AppStrings strings = AppStrings();
    return StringsRepository(strings: strings);
  }

  @override
  ThemeRepository get themeRepository {
    AppColors colors = AppColors();
    AppDimensions dimensions = AppDimensions();
    AppFonts fonts = AppFonts(colors: colors, dimensions: dimensions);

    AppTheme theme = AppTheme(
      colors: colors,
      dimensions: dimensions,
      fonts: fonts,
      materialTheme: ThemeData.fallback()
    );

    return ThemeRepository(theme: theme);
  }

  @override
  UrlRouter get urlRouter => UrlRouter();

  @override
  UserInformationRepository get userInformationRepository => UserInformationRepository();

}