import 'package:flutter/widgets.dart';
import 'package:flutterskel/configuration/ConfigurationRepository.dart' as arch;
import 'package:flutterskel/resources/strings/StringsRepository.dart' as arch;
import 'package:flutterskel/resources/themes/ThemeRepository.dart' as arch;
import 'package:flutterskel/routing/page/PageRouter.dart' as arch;
import 'package:flutterskel/routing/url/UrlRouter.dart' as arch;
import 'package:flutterskel/userinformation/UserInformationRepository.dart' as arch;
import 'package:provider/provider.dart';

abstract class ProviderBuilder<
                  // Data
                  DataSource,
                  // Routing
                  UrlRouter extends arch.UrlRouter,
                  PageRouter extends arch.PageRouter,
                  // Configuration
                  ConfigurationRepository extends arch.ConfigurationRepository,
                  // User Information
                  UserInformationRepository extends arch.UserInformationRepository,
                  // Theme Related: Repository -> Theme -> Fonts -> Dimensions/Colors
                  ThemeRepository extends arch.ThemeRepository,
                  // Strings
                  StringsRepository extends arch.StringsRepository,
                  // Images & Icons
                  Images,
                  Icons> {

  DataSource get dataSource;

  // Routing
  UrlRouter get urlRouter;
  PageRouter get pageRouter;

  ConfigurationRepository get configurationRepository;

  UserInformationRepository get userInformationRepository;

  StringsRepository get stringsRepository;

  // Theme Related
  ThemeRepository get themeRepository;

  Images get images;

  Icons get icons;

  List<SingleChildCloneableWidget> get providers => [
    Provider<DataSource>.value(value: dataSource),
    Provider<UrlRouter>.value(value: urlRouter),
    Provider<PageRouter>.value(value: pageRouter),
    Provider<Images>.value(value: images),
    Provider<Icons>.value(value: icons),

    // Change Notifiers
    ChangeNotifierProvider<ConfigurationRepository>.value(value: configurationRepository),
    ChangeNotifierProvider<UserInformationRepository>.value(value: userInformationRepository),
    ChangeNotifierProvider<StringsRepository>.value(value: stringsRepository),
    ChangeNotifierProvider<ThemeRepository>.value(value: themeRepository)
  ];

  MultiProvider build({@required Widget child}) {
    return MultiProvider(
      providers: providers,
      child: child,
    );
  }
}
