import 'package:flutter/widgets.dart';
import 'package:flutterskel/resources/colors/ColorResources.dart' as arch;
import 'package:flutterskel/resources/dimensions/DimensionResources.dart' as arch;
import 'package:flutterskel/resources/fonts/FontResources.dart' as arch;
import 'package:flutterskel/resources/strings/StringsRepository.dart' as arch;
import 'package:flutterskel/resources/themes/ThemeRepository.dart' as arch;
import 'package:flutterskel/resources/themes/ThemeResources.dart' as arch;
import 'package:flutterskel/routing/page/PageRouter.dart' as arch;
import 'package:flutterskel/routing/url/UrlRouter.dart' as arch;
import 'package:flutterskel/userinformation/UserInformationRepository.dart' as arch;
import 'package:flutterskel/configuration/ConfigurationRepository.dart' as arch;
import 'package:provider/provider.dart';

/// abstract State class
/// This class should only be extended by an abstract class within the app, which fills in all the info except [StateWidget]
/// [StateWidget] should be filled in by the final StatefulWidget class
abstract class RawBaseStatefulWidgetState<
                  StateWidget extends StatefulWidget,
                  // Data
                  DataSource,
                  // Routing
                  UrlRouter extends arch.UrlRouter,
                  PageRouter extends arch.PageRouter,
                  // Configuration
                  ConfigurationRepository extends arch.ConfigurationRepository,
                  Configuration,
                  // User Information
                  UserInformationRepository extends arch.UserInformationRepository,
                  // Themes (Fonts/Dimensions/Colors)
                  ThemeRepository extends arch.ThemeRepository,
                  Theme extends arch.ThemeResources,
                  Colors extends arch.ColorResources,
                  Dimensions extends arch.DimensionResources,
                  Fonts extends arch.FontResources,
                  // Strings
                  StringRepo extends arch.StringsRepository,
                  Strings,
                  // Images and Icons
                  Images,
                  Icons> extends State<StateWidget> {

  // Data
  DataSource get dataSource => Provider.of<DataSource>(context);

  // Routing
  UrlRouter get urlRouter => Provider.of<UrlRouter>(context);
  PageRouter get pageRouter => Provider.of<PageRouter>(context);

  // Configuration
  ConfigurationRepository get configurationRepository => Provider.of<ConfigurationRepository>(context);
  Configuration get configuration => configurationRepository.configuration;

  // User
  UserInformationRepository get userInformation => Provider.of<UserInformationRepository>(context);

  // Theme Stuff
  ThemeRepository get themeRepository => Provider.of<ThemeRepository>(context);
  Theme get theme => themeRepository.theme;
  Colors get colors => theme.colors;
  Dimensions get dimensions => theme.dimensions;
  Fonts get fonts => theme.fonts;

  // Strings
  StringRepo get stringsRepository => Provider.of<StringRepo>(context);
  Strings get strings => stringsRepository.strings;

  // Images and Icons
  Images get images => Provider.of<Images>(context);
  Icons get icons => Provider.of<Icons>(context);
}
