import 'package:flutter/widgets.dart';
import 'package:flutterskel/resources/colors/ColorResources.dart' as arch;
import 'package:flutterskel/resources/dimensions/DimensionResources.dart' as arch;
import 'package:flutterskel/resources/fonts/FontResources.dart' as arch;
import 'package:flutterskel/resources/strings/StringsRepository.dart' as arch;
import 'package:flutterskel/resources/themes/ThemeRepository.dart' as arch;
import 'package:flutterskel/resources/themes/ThemeResources.dart' as arch;
import 'package:flutterskel/routing/page/PageRouter.dart' as arch;
import 'package:flutterskel/routing/url/UrlRouter.dart' as arch;
import 'package:flutterskel/userinformation/UserInformationRepository.dart' as arch;
import 'package:flutterskel/configuration/ConfigurationRepository.dart' as arch;
import 'package:provider/provider.dart';

/// Raw abstract State class
/// This class should only be extended by an abstract class within the app, which fills in all the info except [StateWidget]
/// [StateWidget] should be filled in by the final StatefulWidget class
abstract class RawBaseStatelessWidget<
                  // Data
                  DataSource,
                  // Routing
                  UrlRouter extends arch.UrlRouter,
                  PageRouter extends arch.PageRouter,
                  // Configuration
                  ConfigurationRepository extends arch.ConfigurationRepository,
                  Configuration,
                  // User Information
                  UserInformationRepository extends arch.UserInformationRepository,
                  // Themes (Fonts/Dimensions/Colors)
                  ThemeRepository extends arch.ThemeRepository,
                  Theme extends arch.ThemeResources,
                  Colors extends arch.ColorResources,
                  Dimensions extends arch.DimensionResources,
                  Fonts extends arch.FontResources,
                  // Strings
                  StringRepo extends arch.StringsRepository,
                  Strings,
                  // Images and Icons
                  Images,
                  Icons> extends StatelessWidget {

  // Data
  DataSource getDataSource(BuildContext context) => Provider.of<DataSource>(context);

  // Routing
  UrlRouter urlRouter(BuildContext context) => Provider.of<UrlRouter>(context);
  PageRouter pageRouter(BuildContext context) => Provider.of<PageRouter>(context);

  // Configuration
  ConfigurationRepository getConfigurationRepository(BuildContext context) => Provider.of<ConfigurationRepository>(context);
  Configuration configuration(BuildContext context) => getConfigurationRepository(context).configuration;

  // User Information
  UserInformationRepository userInformation(BuildContext context) => Provider.of<UserInformationRepository>(context);

  // Strings
  StringRepo stringsRepository(BuildContext context) => Provider.of<StringRepo>(context);
  Strings strings(BuildContext context) => stringsRepository(context).strings;

  // Theme
  ThemeRepository themeRepo(BuildContext context) => Provider.of<ThemeRepository>(context);
  Theme theme(BuildContext context) => themeRepo(context).theme;
  Colors colors(BuildContext context) => theme(context).colors;
  Dimensions dimensions(BuildContext context) => theme(context).dimensions;
  Fonts fonts(BuildContext context) => theme(context).fonts;

  // Images and Icons
  Images images(BuildContext context) => Provider.of<Images>(context);
  Icons icons(BuildContext context) => Provider.of<Icons>(context);

}
